
/* Chrysalide - Outil d'analyse de fichiers binaires
 * log.c - équivalent Python du fichier "gui/panels/log.c"
 *
 * Copyright (C) 2009-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "log.h"


#include <pygobject.h>


#include <gui/panels/log.h>


#include "panel.h"
#include "../../helpers.h"



/* Affiche un message dans le journal des messages système. */
static PyObject *py_log_panel_log_message(PyObject *, PyObject *);

/* Définit les constantes pour les types de message. */
static bool define_python_log_constants(PyTypeObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = classe assurant le lien avec l'éditeur de messages.   *
*                args = arguments fournis à l'appel.                          *
*                                                                             *
*  Description : Affiche un message dans le journal des messages système.     *
*                                                                             *
*  Retour      : Rien en équivalent Python.                                   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_log_panel_log_message(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Bilan à retourner           */
    LogMessageType type;                    /* Espèce du message           */
    const char *msg;                        /* Contenu du message          */

    if (!PyArg_ParseTuple(args, "ls", &type, &msg))
        return NULL;

    switch (type)
    {
        case LMT_INFO:
        case LMT_BAD_BINARY:
        case LMT_PROCESS:
        case LMT_ERROR:
        case LMT_WARNING:
            log_simple_message(type, msg);
            result = Py_None;
            Py_INCREF(result);
            break;

        default:
            PyErr_SetString(PyExc_ValueError, 
                            _("Invalid type of message"));
            result = NULL;
            break;

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : obj_type = type dont le dictionnaire est à compléter.        *
*                                                                             *
*  Description : Définit les constantes pour les types de message.            *
*                                                                             *
*  Retour      : true en cas de succès de l'opération, false sinon.           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool define_python_log_constants(PyTypeObject *obj_type)
{
    bool result;                            /* Bilan à retourner           */

    result = true;

    result &= PyDict_AddIntMacro(obj_type, LMT_INFO);
    result &= PyDict_AddIntMacro(obj_type, LMT_BAD_BINARY);
    result &= PyDict_AddIntMacro(obj_type, LMT_PROCESS);
    result &= PyDict_AddIntMacro(obj_type, LMT_ERROR);
    result &= PyDict_AddIntMacro(obj_type, LMT_WARNING);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_log_panel_type(void)
{
    static PyMethodDef py_log_panel_methods[] = {
        {
            "log_message", (PyCFunction)py_log_panel_log_message,
            METH_VARARGS | METH_STATIC,
            "log_message(type, msg, /)\n--\n\nDisplay a message in the log window, if any."
        },
        { NULL }
    };

    static PyGetSetDef py_log_panel_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_log_panel_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.gui.panels.LogPanel",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide log panel",

        .tp_methods     = py_log_panel_methods,
        .tp_getset      = py_log_panel_getseters

    };

    return &py_log_panel_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.gui.panels.LogPanel'.  *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_log_panel(PyObject *module)
{
    PyTypeObject *py_log_panel_type;        /* Type Python 'LoadedBinary'  */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_log_panel_type = get_python_log_panel_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_LOG_PANEL, py_log_panel_type, get_python_panel_item_type()))
        return false;

    if (!define_python_log_constants(py_log_panel_type))
        return false;

    return true;

}
